#!/usr/bin/env python
# -*- coding: utf-8 -*-import socket

# Dataprobe ipIO-8 Control
# 
# Author: David Weiss dweiss@dataprobe.com
# Revision: 1.2  Sequence Number Req
# Date: 7/17/2009
#
# Summary:  Relay control for ipIO-8 Digital I/O devices
# Takes five arguments: <ipaddr> <channel> <state> <userid> <password>
# 
#
# <ipaddr>    IP Address of ipIO-8 unit  - Required
# <channel>   Relay number 1 - 8         - Required
# <state>     0 or 1 for Open or Closed  - Required
# <userid>    User ID  - Not Required and Not implemented for iPIO-8 ver 1.1 
# <password>  Password - Not Required and Not implemented for iPIO-8 ver 1.1  
#

import sys
import re
import socket
import struct

if len(sys.argv) < 4:  # error check for all required arguments
   print "Error:"
   print "3 arguments required, separated by spaces."
   print "ipio_control.py IPAddress channel state"
   print "arguments userid password are optional and not supported as of Ver 1.2"
   sys.exit(1)

ip = sys.argv[1]
ch = int(sys.argv[2])
st = int(sys.argv[3])
uid = 'u'
pas = 'p'
if len(sys.argv) > 4 :
   uid = sys.argv[4]
   pas = sys.argv[5]
okch = [1, 2, 3, 4, 5, 6, 7, 8]
okst = [0, 1]
err = []
p_pas = pas.ljust(21,'P')
p_uid = uid.ljust(21,'U')
hello = 'hello-000'+'\x00'
ch_ar = ['\x00', '\x01', '\x02', '\x03', '\x04', '\x05', '\x06', '\x07', '\x08']
mode_ar = ['\x00', '\x01'] 
#
# This section validates the arguments
#
def ipFormatChk(ip_str): 
   pattern = r"\b(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b" 
   if re.match(pattern, ip_str): 
      return True 
   else: 
      return False
   
if ch not in okch:
   err.append("Channel Error")
if st not in okst:
   err.append("State Error")
if not ipFormatChk(ip):
   err.append("IP Address Format Error")

#
# This section opens the socket and checks for connection
#
   
if ch in okch and st in okst and ipFormatChk(ip):  
   s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
   port = 9100
   try:
      s.connect((ip, port))
   except socket.timeout, msg:
      print "Timeout Connecting to "
      print ip
      s.close()
      sys.exit(1)
   except socket.error, msg:
      print "Error Connecting to "
      print ip
      s.close()
      sys.exit(1)
#
# Send the hello handshake, receive back two byte sequence number
#  
   s.send(hello)
   seq = s.recv(2)
   value = struct.unpack("<h", seq)  #  Turn the two bytes received into an integer
   seq1 = value[0]+1                 #  add 1 to it and turn it back to two bytes 
   seq1a = struct.pack("<h", seq1)   #
#
# Send the I/O Status - Change Relay Command
#
# \x03 = I/O Status Command
# p_uid = User ID padded out to 21 characters
# p_pad = Password padded out to 21 characters
# \x01 = Change Relay command type
# \x00 = Null descriptor for Change Relay command
# seq1a = Sequence number (+1)
# ch_ar[ch] = the channel to be changed.  \x01 for relay 1..... \x08 for relay 8
# mode_ar[st] = the state to be changed to. \x00 for Open, \x01 for Closed
#
   s.send('\x03'+p_uid+p_pas+'\x01\x00'+seq1a+ch_ar[ch]+mode_ar[st])
#
# receive back one byte for ack
   ack = s.recv(1)
   if ack == '\x00':
      print 'OK'
   else:
      print 'ERROR'
# close the socket, we're done.
   s.close()
else:
    while err:
       print(err.pop())