#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Dataprobe ipIO-8 Monitor Program with email
# 
# Author: David Weiss dweiss@dataprobe.com
# Revision: Version 1.2
# Date: 7/17/09
#
# Summary:  Monitors ipIO-8 activity, logs activity and sends one or more
#           emails per input status event
#
# Requires: two .csv files
#           ipio_config.csv  Config file:
#               SMTP Server, From Address
#           ipio_email.csv details for email addresses to be notified:
#               ipAddress, locationID, channel, channelID, emailAddress, openID, closedID
#
# Generates or opens .csv file ipio_log.csv
#           logs all input activity

import socket, select
from time import strftime
import os
import subprocess
import csv
import smtplib

reader = csv.reader(open('ipio_email.csv', 'rb'), dialect=csv.excel)
array = []
for row in reader:
    array.append(row)   
config = csv.reader(open('ipio_config.csv', 'rU'), dialect=csv.excel)
for row in config:
    smtpserv = row[0] 
    fromAddress = row[1]
open_sockets = []
listening_socket = socket.socket( socket.AF_INET, socket.SOCK_STREAM )
listening_socket.bind( ("", 9100) )
listening_socket.listen(25)

# test and open .csv file.  Write header & close
if os.access("ipio_log.csv", os.W_OK): # if the file exists and is closed, create header
    dtstamp = strftime("%Y-%m-%d %H:%M:%S")
    ipio_header = dtstamp+",Program Start,,,\n"
else:
    if not os.path.isfile("p:\ipio_log.csv"): # if the file does not exist, build first line
        ipio_header = "Time,Location,Channel,Status\n"
    else:
        print "file ipio_log.csv is open. Close it please."  # if file is open, Fail
        f = raw_input("press any key")
ipiolog = open("ipio_log.csv", "a")
ipiolog.write(ipio_header)
ipiolog.close()              
while True:
    # Waits for I/O being available for reading from any socket object.
    rlist, wlist, xlist = select.select( [listening_socket] + open_sockets, [], [] )
    for i in rlist:
        if i is listening_socket:
            new_socket, addr = listening_socket.accept()
            open_sockets.append(new_socket)
        else:
            data = i.recv(1024)
            if data == "":
                open_sockets.remove(i)
            elif data[0:9] == 'hello-000':
                i.send('\x1f\x0e') # pseudo sequence number. could be incremented
            else:
                i.send('\x00')              # reply with ack (null)
                dtstamp = strftime("%Y-%m-%d %H:%M:%S")
                dt =  strftime("%Y-%m-%d")
                tm = strftime("%H:%M:%S")
                add_r = repr(addr)[2:15]
                ch1 = data[0]
                # link OK message is optional.  Comment out print statement below if desired
                if ch1 == "\x04":
                    print dtstamp +"  "+add_r+ "  Link OK"                    
                else:
                    # These two bytes hold the channel and status information
                    chan = data[47]
                    inst = data[48]
                    ch_ar = ['\x00', '\x01', '\x02', '\x03', '\x04', '\x05', '\x06', '\x07', '\x08']
                    chan_r = ch_ar.index(chan)
                    stat_r = ch_ar.index(inst)
                    if stat_r == 1:
                        dispstat = 6
                        status = "Closed"
                    else:
                        status = "Open  "
                        dispstat = 5
                    # print status message
                    print dtstamp +"  "+add_r+ "  Ch " + repr(chan_r)+" = " + status
                    # write data to .csv file
                    csv_line = dtstamp+","+add_r+","+repr(chan_r)+","+status+"\n"
                    ipiolog = open("ipio_log.csv", "a")
                    ipiolog.write(csv_line)
                    ipiolog.close()
                    
                    # do lookup of csv array
                    
                    for row in array:
                        # key, ipAddress, locationID, channel, channelID, emailAddress
                        if row[0]==add_r and row[2]==repr(chan_r):
                            print 'Email sent to ' + row[4]
                        #email sending starts here                        
                        
                            msg = "Subject: ipIO Status Update\n\n At: "+dtstamp+"\n Site:    "+row[1]+ "    ("+add_r+")\n Channel: "+row[3]+"    ("+repr(chan_r)+")\n Status:  "+row[dispstat]+'    ('+status+')'
                            server = smtplib.SMTP(smtpserv, 25)
                            server.sendmail(fromAddress, row[4], msg)